@@ -56,6 +56,9 @@ class ConfigController extends ActionController
                 $this->setAutorefreshInterval(20);
             }
         } catch (Exception $e) {
+            $this->content()->prepend(
+                Html::tag('p', ['class' => 'warning'], $e->getMessage())
+            );
             // No problem, Icinga might be reloading
         }
 
@@ -104,7 +107,7 @@ class ConfigController extends ActionController
         // TODO: require POST
         $checksum = $this->params->get('checksum');
         if ($checksum) {
-            $config = IcingaConfig::load(Util::hex2binary($checksum), $this->db());
+            $config = IcingaConfig::load(hex2bin($checksum), $this->db());
         } else {
             $config = IcingaConfig::generate($this->db());
             $checksum = $config->getHexChecksum();
@@ -234,7 +237,7 @@ class ConfigController extends ActionController
         }
         $this->assertPermission('director/showconfig');
         $config = IcingaConfig::load(
-            Util::hex2binary($this->params->getRequired('checksum')),
+            hex2bin($this->params->getRequired('checksum')),
             $this->db()
         );
         $deploymentId = $this->params->get('deployment_id');
@@ -293,7 +296,7 @@ class ConfigController extends ActionController
             $this->addBackLink('director/config/files', $params);
         }
 
-        $config = IcingaConfig::load(Util::hex2binary($this->params->get('config_checksum')), $this->db());
+        $config = IcingaConfig::load(hex2bin($this->params->get('config_checksum')), $this->db());
         $this->addTitle($this->translate('Config file "%s"'), $filename);
         $this->content()->add(new ShowConfigFile(
             $config->getFile($filename),
@@ -310,7 +313,12 @@ class ConfigController extends ActionController
     public function storeAction()
     {
         $this->assertPermission('director/deploy');
-        $config = IcingaConfig::generate($this->db());
+        try {
+            $config = IcingaConfig::generate($this->db());
+        } catch (Exception $e) {
+            Notification::error($e->getMessage());
+            $this->redirectNow('director/config/deployments');
+        }
         $this->redirectNow(
             Url::fromPath(
                 'director/config/files',
@@ -343,7 +351,8 @@ class ConfigController extends ActionController
             }
         }
 
-        $this->content()->add(Html::tag('form', ['action' => $this->url(), 'method' => 'GET'], [
+        $baseUrl = $this->url()->without(['left', 'right']);
+        $this->content()->add(Html::tag('form', ['action' => (string) $baseUrl, 'method' => 'GET'], [
             new HtmlString($this->view->formSelect(
                 'left',
                 $leftSum,
@@ -352,7 +361,7 @@ class ConfigController extends ActionController
             )),
             Link::create(
                 Icon::create('flapping'),
-                $this->url(),
+                $baseUrl,
                 ['left' => $rightSum, 'right' => $leftSum]
             ),
             new HtmlString($this->view->formSelect(
@@ -386,8 +395,8 @@ class ConfigController extends ActionController
         $rightSum = $p->getRequired('right');
         $filename = $p->getRequired('file_path');
 
-        $left = IcingaConfig::load(Util::hex2binary($leftSum), $db);
-        $right = IcingaConfig::load(Util::hex2binary($rightSum), $db);
+        $left = IcingaConfig::load(hex2bin($leftSum), $db);
+        $right = IcingaConfig::load(hex2bin($rightSum), $db);
 
         $this
             ->addTitle($this->translate('Config file "%s"'), $filename)
