@@ -2,7 +2,10 @@
 
 namespace Icinga\Module\Director\Controllers;
 
+use dipl\Html\Html;
+use Icinga\Module\Director\Web\Widget\HealthCheckPluginOutput;
 use Icinga\Module\Director\Dashboard\Dashboard;
+use Icinga\Module\Director\Health;
 use Icinga\Module\Director\Web\Controller\ActionController;
 use Icinga\Module\Director\Web\Form\DbSelectorForm;
 
@@ -22,17 +25,13 @@ class DashboardController extends ActionController
         }
     }
 
-    /**
-     * @throws \Icinga\Exception\ConfigurationError
-     * @throws \Icinga\Exception\Http\HttpNotFoundException
-     */
     public function indexAction()
     {
         if ($this->getRequest()->isGet()) {
             $this->setAutorefreshInterval(10);
         }
 
-        $mainDashboards = ['Objects', 'Alerts', 'Automation', 'Deployment', 'Data'];
+        $mainDashboards = ['Objects', 'Alerts', 'Automation', 'Deployment', 'Director', 'Data'];
         $this->setTitle($this->translate('Icinga Director - Main Dashboard'));
         $names = $this->params->getValues('name', $mainDashboards);
         if (! $this->params->has('name')) {
@@ -43,7 +42,22 @@ class DashboardController extends ActionController
             $dashboard = Dashboard::loadByName($name, $this->db());
             $this->tabs($dashboard->getTabs())->activate($name);
         } else {
-            $this->addSingleTab($this->translate('Overview'));
+            $this->tabs()->add('main', [
+                'label' => $this->translate('Overview'),
+                'url' => 'director'
+            ])->activate('main');
+            if ($this->hasPermission('director/admin')) {
+                $this->tabs()->add('health', [
+                    'label' => $this->translate('Health'),
+                    'url' => 'director/health'
+                ]);
+                $state = $this->getHealthState();
+                if ($state->isProblem()) {
+                    $this->tabs()->get('health')->setTagParams([
+                        'class' => 'state-' . strtolower($state->getName())
+                    ]);
+                }
+            }
         }
 
         $cntDashboards = 0;
@@ -66,4 +80,16 @@ class DashboardController extends ActionController
             $this->content()->add($msg);
         }
     }
+
+    /**
+     * @return \Icinga\Module\Director\CheckPlugin\PluginState
+     */
+    protected function getHealthState()
+    {
+        $health = new Health();
+        $health->setDbResourceName($this->getDbResourceName());
+        $output = new HealthCheckPluginOutput($health);
+
+        return $output->getState();
+    }
 }
