@@ -2,6 +2,10 @@
 
 namespace Icinga\Module\Director\Controllers;
 
+use dipl\Html\Html;
+use dipl\Html\Link;
+use dipl\Web\Url;
+use dipl\Web\Widget\Tabs;
 use Exception;
 use Icinga\Module\Director\CustomVariable\CustomVariableDictionary;
 use Icinga\Module\Director\Db\AppliedServiceSetLoader;
@@ -19,16 +23,26 @@ use Icinga\Module\Director\Web\Table\IcingaHostAppliedForServiceTable;
 use Icinga\Module\Director\Web\Table\IcingaHostAppliedServicesTable;
 use Icinga\Module\Director\Web\Table\IcingaHostServiceTable;
 use Icinga\Module\Director\Web\Table\IcingaServiceSetServiceTable;
-use Icinga\Web\Url;
-use dipl\Html\Link;
+use Icinga\Module\Director\Web\Widget\HostServiceRedirector;
 
 class HostController extends ObjectController
 {
     protected function checkDirectorPermissions()
     {
-        $this->assertPermission('director/hosts');
+        if (in_array($this->getRequest()->getActionName(), [
+            'servicesro',
+            'findservice',
+            'invalidservice'
+        ])) {
+            $this->assertPermission('director/monitoring/services-ro');
+        } else {
+            $this->assertPermission('director/hosts');
+        }
     }
 
+    /**
+     * @return HostgroupRestriction
+     */
     protected function getHostgroupRestriction()
     {
         return new HostgroupRestriction($this->db(), $this->Auth());
@@ -85,6 +99,36 @@ class HostController extends ObjectController
         ));
     }
 
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public function findserviceAction()
+    {
+        $host = $this->getHostObject();
+        $redirector = new HostServiceRedirector($host, $this->getAuth());
+        $this->redirectNow(
+            $redirector->getRedirectionUrl($this->params->get('service'))
+        );
+    }
+
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public function invalidserviceAction()
+    {
+        $this->content()->add(
+            Html::tag('p', ['class' => 'error'], sprintf(
+                $this->translate('No such service: %s'),
+                $this->params->get('service')
+            ))
+        );
+
+        $this->servicesAction();
+    }
+
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function servicesAction()
     {
         $this->addServicesHeader();
@@ -149,7 +193,98 @@ class HostController extends ObjectController
         }
     }
 
-    protected function addHostServiceSetTables(IcingaHost $host, IcingaHost $affectedHost = null)
+    /**
+     * Hint: this duplicates quite some logic from servicesAction. We might want
+     *       to clean this up, but as soon as we store fully resolved Services this
+     *       will be obsolete anyways
+     *
+     * @throws \Icinga\Exception\NotFoundError
+     * @throws \Icinga\Security\SecurityException
+     * @throws \Icinga\Exception\MissingParameterException
+     */
+    public function servicesroAction()
+    {
+        $this->assertPermission('director/monitoring/services-ro');
+        $host = $this->getHostObject();
+        $service = $this->params->getRequired('service');
+        $db = $this->db();
+        $this->controls()->setTabs(new Tabs());
+        $this->addSingleTab($this->translate('Configuration (read-only)'));
+        $this->addTitle($this->translate('Services on %s'), $host->getObjectName());
+        $content = $this->content();
+        $table = IcingaHostServiceTable::load($host)
+            ->setReadonly()
+            ->highlightService($service)
+            ->setTitle($this->translate('Individual Service objects'));
+
+        if (count($table)) {
+            $content->add($table);
+        }
+
+        if ($applied = $host->vars()->get($db->settings()->magic_apply_for)) {
+            if ($applied instanceof CustomVariableDictionary) {
+                $table = IcingaHostAppliedForServiceTable::load($host, $applied)
+                    ->setReadonly()
+                    ->highlightService($service)
+                    ->setTitle($this->translate('Generated from host vars'));
+                if (count($table)) {
+                    $content->add($table);
+                }
+            }
+        }
+
+        /** @var IcingaHost[] $parents */
+        $parents = IcingaTemplateRepository::instanceByObject($this->object)
+            ->getTemplatesFor($this->object, true);
+        foreach ($parents as $parent) {
+            $table = IcingaHostServiceTable::load($parent)
+                ->setReadonly()
+                ->highlightService($service)
+                ->setInheritedBy($host);
+            if (count($table)) {
+                $content->add(
+                    $table->setTitle(sprintf(
+                        'Inherited from %s',
+                        $parent->getObjectName()
+                    ))
+                );
+            }
+        }
+
+        $this->addHostServiceSetTables($host);
+        foreach ($parents as $parent) {
+            $this->addHostServiceSetTables($parent, $host, $service);
+        }
+
+        $appliedSets = AppliedServiceSetLoader::fetchForHost($host);
+        foreach ($appliedSets as $set) {
+            $title = sprintf($this->translate('%s (Applied Service set)'), $set->getObjectName());
+
+            $content->add(
+                IcingaServiceSetServiceTable::load($set)
+                    // ->setHost($host)
+                    ->setAffectedHost($host)
+                    ->setReadonly()
+                    ->highlightService($service)
+                    ->setTitle($title)
+            );
+        }
+
+        $table = IcingaHostAppliedServicesTable::load($host)
+            ->setReadonly()
+            ->highlightService($service)
+            ->setTitle($this->translate('Applied services'));
+
+        if (count($table)) {
+            $content->add($table);
+        }
+    }
+
+    /**
+     * @param IcingaHost $host
+     * @param IcingaHost|null $affectedHost
+     */
+    protected function addHostServiceSetTables(IcingaHost $host, IcingaHost $affectedHost = null, $roService = null)
     {
         $db = $this->db();
         if ($affectedHost === null) {
@@ -174,15 +309,20 @@ class HostController extends ObjectController
         /** @var IcingaServiceSet $set*/
         foreach ($sets as $name => $set) {
             $title = sprintf($this->translate('%s (Service set)'), $name);
-            $this->content()->add(
-                IcingaServiceSetServiceTable::load($set)
-                    ->setHost($host)
-                    ->setAffectedHost($affectedHost)
-                    ->setTitle($title)
-            );
+            $table = IcingaServiceSetServiceTable::load($set)
+                ->setHost($host)
+                ->setAffectedHost($affectedHost)
+                ->setTitle($title);
+            if ($roService) {
+                $table->setReadonly()->highlightService($roService);
+            }
+            $this->content()->add($table);
         }
     }
 
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function appliedserviceAction()
     {
         $db = $this->db();
@@ -216,6 +356,9 @@ class HostController extends ObjectController
         $this->commonForServices();
     }
 
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function inheritedserviceAction()
     {
         $db = $this->db();
@@ -252,6 +395,9 @@ class HostController extends ObjectController
         $this->commonForServices();
     }
 
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function removesetAction()
     {
         // TODO: clean this up, use POST
@@ -276,18 +422,31 @@ class HostController extends ObjectController
         );
     }
 
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function servicesetserviceAction()
     {
         $db = $this->db();
         $host = $this->getHostObject();
         $serviceName = $this->params->get('service');
-        $set = IcingaServiceSet::load($this->params->get('set'), $db);
+        $setParams = [
+            'object_name' => $this->params->get('set'),
+            'host_id'     => $host->get('id')
+        ];
+        $setTemplate = IcingaServiceSet::load($this->params->get('set'), $db);
+        if (IcingaServiceSet::exists($setParams, $db)) {
+            $set = IcingaServiceSet::load($setParams, $db);
+        } else {
+            $set = $setTemplate;
+        }
 
         $service = IcingaService::load([
             'object_name'    => $serviceName,
-            'service_set_id' => $set->get('id')
+            'service_set_id' => $setTemplate->get('id')
         ], $this->db());
         $service = IcingaService::create([
+            'id'          => $service->get('id'),
             'object_type' => 'apply',
             'object_name' => $serviceName,
             'host_id'     => $host->get('id'),
@@ -326,6 +485,9 @@ class HostController extends ObjectController
         $this->tabs()->activate('services');
     }
 
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function agentAction()
     {
         $selfService = new SelfService($this->getHostObject(), $this->api());
@@ -356,12 +518,36 @@ class HostController extends ObjectController
                         'data-base-target' => '_next'
                     ]
                 ));
+
+                // Intentionally placed here, show it only for deployed Hosts
+                $this->addOptionalInspectLink();
             }
         } catch (Exception $e) {
             // Silently ignore errors in the monitoring module
         }
     }
 
+    protected function addOptionalInspectLink()
+    {
+        if (! $this->hasPermission('director/inspect')) {
+            return;
+        }
+
+        $this->actions()->add(Link::create(
+            $this->translate('Inspect'),
+            'director/inspect/object',
+            [
+                'type'   => 'host',
+                'plural' => 'hosts',
+                'name'   => $this->object->getObjectName()
+            ],
+            [
+                'class'            => 'icon-zoom-in',
+                'data-base-target' => '_next'
+            ]
+        ));
+    }
+
     /**
      * @return IcingaHost
      */
