@@ -3,13 +3,15 @@
 namespace Icinga\Module\Director\Controllers;
 
 use Exception;
+use Icinga\Module\Director\Forms\KickstartForm;
 
 class KickstartController extends DashboardController
 {
     public function indexAction()
     {
-        $this->singleTab($this->view->title = $this->translate('Kickstart'));
-        $form = $this->view->form = $this->loadForm('kickstart');
+        $this->addSingleTab($this->translate('Kickstart'))
+            ->addTitle($this->translate('Director Kickstart Wizard'));
+        $form = KickstartForm::load();
         try {
             $form->setEndpoint($this->db()->getDeploymentEndpoint());
         } catch (Exception $e) {
@@ -17,5 +19,6 @@ class KickstartController extends DashboardController
         }
 
         $form->handleRequest();
+        $this->content()->add($form);
     }
 }
