@@ -55,6 +55,17 @@ class IcingaCloneObjectForm extends DirectorForm
             ], 'y');
         }
 
+        if ($this->object instanceof IcingaService && $this->object->get('service_set_id') !== null) {
+            $this->addElement('select', 'target_service_set', [
+                'label'       => $this->translate('Target Service Set'),
+                'description' => $this->translate(
+                    'Clone this service to the very same or to another Service Set'
+                ),
+                'multiOptions' => $this->enumServiceSets(),
+                'value' => $this->object->get('service_set_id')
+            ]);
+        }
+
         if ($this->object->isTemplate() && $this->object->supportsFields()) {
             $this->addBoolean('clone_fields', [
                 'label'       => $this->translate('Clone Template Fields'),
@@ -103,6 +114,13 @@ class IcingaCloneObjectForm extends DirectorForm
             $new->set('object_type', 'object');
         }
 
+        if ($set = $this->getValue('target_service_set')) {
+            $new->set(
+                'service_set_id',
+                IcingaServiceSet::loadWithAutoIncId((int) $set, $connection)->get('id')
+            );
+        }
+
         $services = [];
         $sets = [];
         if ($object instanceof IcingaHost) {
@@ -172,6 +190,16 @@ class IcingaCloneObjectForm extends DirectorForm
         }
     }
 
+    protected function enumServiceSets()
+    {
+        $db = $this->object->getConnection()->getDbAdapter();
+        return $db->fetchPairs(
+            $db->select()
+                ->from('icinga_service_set', ['id', 'object_name'])
+                ->order('object_name')
+        );
+    }
+
     public function setObject(IcingaObject $object)
     {
         $this->object = $object;
