@@ -2,7 +2,6 @@
 
 namespace Icinga\Module\Director\Forms;
 
-use Icinga\Module\Director\Objects\IcingaService;
 use Icinga\Module\Director\Web\Form\DirectorObjectForm;
 
 class IcingaServiceGroupForm extends DirectorObjectForm
@@ -11,11 +10,11 @@ class IcingaServiceGroupForm extends DirectorObjectForm
     {
         $this->addHidden('object_type', 'object');
 
-        $this->addElement('text', 'object_name', array(
+        $this->addElement('text', 'object_name', [
             'label'       => $this->translate('Servicegroup'),
             'required'    => true,
             'description' => $this->translate('Icinga object name for this service group')
-        ));
+        ]);
 
         $this->addGroupDisplayNameElement()
              ->addAssignmentElements()
@@ -24,14 +23,14 @@ class IcingaServiceGroupForm extends DirectorObjectForm
 
     protected function addAssignmentElements()
     {
-        $this->addAssignFilter(array(
-            'columns' => IcingaService::enumProperties($this->db, 'service.'),
+        $this->addAssignFilter([
+            'suggestionContext' => 'ServiceFilterColumns',
             'required' => true,
             'description' => $this->translate(
                 'This allows you to configure an assignment filter. Please feel'
                 . ' free to combine as many nested operators as you want'
             )
-        ));
+        ]);
 
         return $this;
     }
