@@ -16,6 +16,8 @@ class CoreApi implements DeploymentApiInterface
 {
     protected $client;
 
+    protected $initialized = false;
+
     /** @var Db */
     protected $db;
 
@@ -31,6 +33,55 @@ class CoreApi implements DeploymentApiInterface
         return $this;
     }
 
+    /**
+     * @return string|null
+     */
+    public function getVersion()
+    {
+        return $this->parseVersion($this->getRawVersion());
+    }
+
+    public function enableWorkaroundForConnectionIssues()
+    {
+        $version = $this->getVersion();
+
+        if (version_compare($version, '2.8.2', '>=')
+            && version_compare($version, '2.10.2', '<')
+        ) {
+            $this->client->disconnect();
+            $this->client->setKeepAlive(false);
+        }
+    }
+
+    /**
+     * @return string|null
+     */
+    public function getRawVersion()
+    {
+        try {
+            return $this->client()->get('')->getRaw('version');
+        } catch (Exception $exception) {
+            return null;
+        }
+    }
+
+    /**
+     * @param $version
+     * @return string|null
+     */
+    protected function parseVersion($version)
+    {
+        if ($version === null) {
+            return null;
+        }
+
+        if (preg_match('/^v?(\d\.\d+\.\d+)/', $version, $match)) {
+            return $match[1];
+        } else {
+            return null;
+        }
+    }
+
     public function getObjects($pluralType, $attrs = array(), $ignorePackage = null)
     {
         $params = (object) [];
@@ -42,7 +93,7 @@ class CoreApi implements DeploymentApiInterface
             $params->attrs = $attrs;
         }
 
-        return $this->client->get(
+        return $this->client()->get(
             'objects/' . urlencode(strtolower($pluralType)),
             $params
         )->getResult('name');
@@ -51,6 +102,7 @@ class CoreApi implements DeploymentApiInterface
     public function onEvent($callback, $raw = false)
     {
         $this->client->onEvent($callback, $raw);
+
         return $this;
     }
 
@@ -63,7 +115,7 @@ class CoreApi implements DeploymentApiInterface
             $params->attrs = $attrs;
         }
         $url = 'objects/' . urlencode(strtolower($pluraltype)) . '/' . rawurlencode($name) . '?all_joins=1';
-        $res = $this->client->get($url, $params)->getResult('name');
+        $res = $this->client()->get($url, $params)->getResult('name');
 
         // TODO: check key, throw
         return $res[$name];
@@ -83,7 +135,8 @@ class CoreApi implements DeploymentApiInterface
     public function checkHostNow($host)
     {
         $filter = 'host.name == "' . $host . '"';
-        return $this->client->post(
+
+        return $this->client()->post(
             'actions/reschedule-check?filter=' . rawurlencode($filter),
             (object) array(
                 'type' => 'Host'
@@ -94,7 +147,7 @@ class CoreApi implements DeploymentApiInterface
     public function checkServiceNow($host, $service)
     {
         $filter = 'host.name == "' . $host . '" && service.name == "' . $service . '"';
-        $this->client->post(
+        $this->client()->post(
             'actions/reschedule-check?filter=' . rawurlencode($filter),
             (object) array(
                 'type' => 'Service'
@@ -105,7 +158,7 @@ class CoreApi implements DeploymentApiInterface
     public function acknowledgeHostProblem($host, $author, $comment)
     {
         $filter = 'host.name == "' . $host . '"';
-        return $this->client->post(
+        return $this->client()->post(
             'actions/acknowledge-problem?type=Host&filter=' . rawurlencode($filter),
             (object) array(
                 'author'  => $author,
@@ -117,7 +170,7 @@ class CoreApi implements DeploymentApiInterface
     public function removeHostAcknowledgement($host)
     {
         $filter = 'host.name == "' . $host . '"';
-        return $this->client->post(
+        return $this->client()->post(
             'actions/remove-acknowledgement?type=Host&filter=' . rawurlencode($filter)
         );
     }
@@ -125,7 +178,7 @@ class CoreApi implements DeploymentApiInterface
     public function reloadNow()
     {
         try {
-            $this->client->post('actions/restart-process');
+            $this->client()->post('actions/restart-process');
 
             return true;
         } catch (Exception $e) {
@@ -293,7 +346,7 @@ constants
 
     public function runConsoleCommand($command)
     {
-        return $this->client->post(
+        return $this->client()->post(
             'console/execute-script',
             array('command' => $command)
         );
@@ -311,18 +364,18 @@ constants
 
     public function getTypes()
     {
-        return $this->client->get('types')->getResult('name');
+        return $this->client()->get('types')->getResult('name');
     }
 
     public function getType($type)
     {
-        $res = $this->client->get('types', array('name' => $type))->getResult('name');
+        $res = $this->client()->get('types', array('name' => $type))->getResult('name');
         return $res[$type]; // TODO: error checking
     }
 
     public function getStatus()
     {
-        return $this->client->get('status')->getResult('name');
+        return $this->client()->get('status')->getResult('name');
     }
 
     public function listObjects($type, $pluralType)
@@ -330,7 +383,7 @@ constants
         // TODO: more abstraction needed
         // TODO: autofetch and cache pluraltypes
         try {
-            $result = $this->client->get(
+            $result = $this->client()->get(
                 'objects/' . $pluralType,
                 array(
                     'attrs' => array('__name')
@@ -345,7 +398,7 @@ constants
 
     public function getPackages()
     {
-        return $this->client->get('config/packages')->getResult('name');
+        return $this->client()->get('config/packages')->getResult('name');
     }
 
     public function getActiveStageName()
@@ -462,7 +515,7 @@ constants
 
     public function getHostObjects()
     {
-        return $this->getDirectorObjects('Host', 'hosts', [
+        $params = [
             'display_name'          => 'display_name',
             'address'               => 'address',
             'address6'              => 'address6',
@@ -481,7 +534,6 @@ constants
             'enable_flapping'       => 'enable_flapping',
             'enable_perfdata'       => 'enable_perfdata',
             'event_command'         => 'event_command',
-            'flapping_threshold'    => 'flapping_threshold',
             'volatile'              => 'volatile',
             'zone'                  => 'zone',
             'command_endpoint'      => 'command_endpoint',
@@ -490,7 +542,14 @@ constants
             'action_url'            => 'action_url',
             'icon_image'            => 'icon_image',
             'icon_image_alt'        => 'icon_image_alt',
-        ]);
+        ];
+
+        if (version_compare($this->getVersion(), '2.8.0', '>=')) {
+            $params['flapping_threshold_high'] = 'flapping_threshold_high';
+            $params['flapping_threshold_low'] = 'flapping_threshold_low';
+        }
+
+        return $this->getDirectorObjects('Host', 'hosts', $params);
     }
 
     public function getHostGroupObjects()
@@ -581,9 +640,12 @@ constants
     public function collectLogFiles(Db $db)
     {
         $existing = $this->listPackageStages($this->getPackageName());
+        $missing = [];
+        $empty = [];
         foreach (DirectorDeploymentLog::getUncollected($db) as $deployment) {
             $stage = $deployment->get('stage_name');
             if (! in_array($stage, $existing)) {
+                $missing[] = $deployment;
                 continue;
             }
 
@@ -607,12 +669,29 @@ constants
                 ));
             } else {
                 // Stage seems to be incomplete, let's try again next time
+                $empty[] = $deployment;
                 continue;
             }
             $deployment->set('stage_collected', 'y');
 
             $deployment->store();
         }
+
+        foreach ($missing as $deployment) {
+            $deployment->set('stage_collected', 'n');
+            $deployment->store();
+        }
+
+        $running = DirectorDeploymentLog::getRelatedToActiveStage($this, $db);
+        if ($running !== null) {
+            foreach ($empty as $deployment) {
+                if ($deployment->get('start_time') < $running->get('start_time')) {
+                    $deployment->set('stage_collected', 'n');
+                    $deployment->store();
+                    $this->deleteStage($this->getPackageName(), $deployment->get('stage_name'));
+                }
+            }
+        }
     }
 
     public function wipeInactiveStages(Db $db)
@@ -623,14 +702,14 @@ constants
             if (array_key_exists($stage, $uncollected)) {
                 continue;
             }
-            $this->client->delete('config/stages/' . $packageName . '/' . $stage);
+            $this->client()->delete('config/stages/' . $packageName . '/' . $stage);
         }
     }
 
     public function listStageFiles($stage)
     {
         return array_keys(
-            $this->client->get(sprintf(
+            $this->client()->get(sprintf(
                 'config/stages/%s/%s',
                 urlencode($this->getPackageName()),
                 urlencode($stage)
@@ -640,7 +719,7 @@ constants
 
     public function getStagedFile($stage, $file)
     {
-        return $this->client->getRaw(sprintf(
+        return $this->client()->getRaw(sprintf(
             'config/files/%s/%s/%s',
             urlencode($this->getPackageName()),
             urlencode($stage),
@@ -656,12 +735,12 @@ constants
 
     public function createPackage($name)
     {
-        return $this->client->post('config/packages/' . urlencode($name))->succeeded();
+        return $this->client()->post('config/packages/' . urlencode($name))->succeeded();
     }
 
     public function deletePackage($name)
     {
-        return $this->client->delete('config/packages/' . urlencode($name))->succeeded();
+        return $this->client()->delete('config/packages/' . urlencode($name))->succeeded();
     }
 
     public function assertPackageExists($name)
@@ -680,10 +759,11 @@ constants
 
     public function deleteStage($packageName, $stageName)
     {
-        return $this->client->delete('config/stages', [
-            'module' => $packageName,
-            'stage'  => $stageName
-        ])->succeeded();
+        $this->client()->delete(sprintf(
+            'config/stages/%s/%s',
+            rawurlencode($packageName),
+            rawurlencode($stageName)
+        ))->succeeded();
     }
 
     /**
@@ -708,7 +788,7 @@ constants
 
         $url = sprintf('events?queue=%s&types=%s', $queue, implode('&types=', $allTypes));
 
-        $this->client->request('post', $url, null, false, true);
+        $this->client()->request('post', $url, null, false, true);
     }
 
     /**
@@ -738,7 +818,7 @@ constants
 
         $this->assertPackageExists($packageName);
 
-        $response = $this->client->post('config/stages/' . urlencode($packageName), [
+        $response = $this->client()->post('config/stages/' . urlencode($packageName), [
             'files' => $config->getFileContents()
         ]);
 
@@ -784,4 +864,14 @@ constants
             $logLen - (strlen($begin) + strlen($end))
         ) . $end;
     }
+
+    protected function client()
+    {
+        if ($this->initialized === false) {
+            $this->initialized = true;
+            $this->enableWorkaroundForConnectionIssues();
+        }
+
+        return $this->client;
+    }
 }
