@@ -42,15 +42,12 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
         $parts = array();
         $where = $db->quoteInto('varname = ?', $varname);
         foreach (static::$allTables as $table) {
-            $parts[] = sprintf(
-                'SELECT COUNT(*) as cnt FROM ' . $table . ' WHERE %s',
-                $where
-            );
+            $parts[] = "SELECT COUNT(*) as cnt FROM $table WHERE $where";
         }
 
-        $query = 'SELECT SUM(cnt) AS cnt FROM ('
-            . implode(' UNION ALL ', $parts)
-            . ') sub';
+        $sub = implode(' UNION ALL ', $parts);
+        $query = "SELECT SUM(sub.cnt) AS cnt FROM ($sub) sub";
+
         return (int) $db->fetchOne($query);
     }
 
@@ -278,6 +275,32 @@ class CustomVariables implements Iterator, Countable, IcingaConfigRenderer
         return $this;
     }
 
+    public function restoreStoredVar($key)
+    {
+        if (array_key_exists($key, $this->storedVars)) {
+            $this->vars[$key] = clone($this->storedVars[$key]);
+            $this->vars[$key]->setUnmodified();
+            $this->recheckForModifications();
+            $this->refreshIndex();
+        } elseif (array_key_exists($key, $this->vars)) {
+            unset($this->vars[$key]);
+            $this->recheckForModifications();
+            $this->refreshIndex();
+        }
+    }
+
+    protected function recheckForModifications()
+    {
+        $this->modified = false;
+        foreach ($this->vars as $var) {
+            if ($var->hasBeenModified()) {
+                $this->modified = true;
+
+                return;
+            }
+        }
+    }
+
     public function getOriginalVars()
     {
         return $this->storedVars;
