@@ -11,7 +11,6 @@ use Icinga\Module\Director\Data\Db\DbConnection;
 use Icinga\Module\Director\Objects\IcingaEndpoint;
 use Icinga\Module\Director\Objects\IcingaObject;
 use RuntimeException;
-use Zend_Db_Expr;
 use Zend_Db_Select;
 
 class Db extends DbConnection
@@ -31,6 +30,11 @@ class Db extends DbConnection
         return $this->getDbAdapter();
     }
 
+    /**
+     * @param $callable
+     * @return $this
+     * @throws Exception
+     */
     public function runFailSafeTransaction($callable)
     {
         if (! is_callable($callable)) {
@@ -43,7 +47,11 @@ class Db extends DbConnection
             $callable();
             $db->commit();
         } catch (Exception $e) {
-            $db->rollback();
+            try {
+                $db->rollback();
+            } catch (Exception $e) {
+                // Well... there is nothing we can do here.
+            }
             throw $e;
         }
 
@@ -292,7 +300,7 @@ class Db extends DbConnection
         $result = $this->db()->fetchOne($sql);
 
         if ($binary) {
-            return Util::hex2binary($result);
+            return hex2bin($result);
         } else {
             return $result;
         }
@@ -323,7 +331,7 @@ class Db extends DbConnection
         );
 
         return $db->fetchRow(
-            $db->quoteInto($sql, $this->quoteBinary(Util::hex2binary($checksum)))
+            $db->quoteInto($sql, $this->quoteBinary(hex2bin($checksum)))
         );
     }
 
@@ -702,15 +710,6 @@ class Db extends DbConnection
         }
     }
 
-    public function quoteBinary($binary)
-    {
-        if ($this->isPgsql()) {
-            return new Zend_Db_Expr("'\\x" . bin2hex($binary) . "'");
-        }
-
-        return $binary;
-    }
-
     public function enumDeployedConfigs()
     {
         $db = $this->db();
