@@ -138,7 +138,7 @@ class IcingaConfig
 
     public function getHexChecksum()
     {
-        return Util::binary2hex($this->checksum);
+        return bin2hex($this->checksum);
     }
 
     /**
@@ -204,7 +204,7 @@ class IcingaConfig
             array('checksum' => $connection->dbHexFunc('c.checksum'))
         )->where(
             'checksum = ?',
-            $connection->quoteBinary(Util::hex2binary($checksum))
+            $connection->quoteBinary(hex2bin($checksum))
         );
 
         return $db->fetchOne($query) === $checksum;
@@ -222,7 +222,7 @@ class IcingaConfig
             array()
         )->where(
             'last_activity_checksum = ?',
-            $connection->quoteBinary(Util::hex2binary($checksum))
+            $connection->quoteBinary(hex2bin($checksum))
         )->order('l.id DESC')->limit(1);
 
         return self::load($db->fetchOne($query), $connection);
@@ -240,7 +240,7 @@ class IcingaConfig
             array()
         )->where(
             'last_activity_checksum = ?',
-            $connection->quoteBinary(Util::hex2binary($checksum))
+            $connection->quoteBinary(hex2bin($checksum))
         )->order('l.id DESC')->limit(1);
 
         return $db->fetchOne($query) === $checksum;
@@ -534,7 +534,7 @@ globals.directorWarnOnceForThresholds = function() {
         return sprintf(
             '
 const DirectorOverrideTemplate = "%s"
-if (! globals[DirectorOverrideTemplate]) {
+if (! globals.contains(DirectorOverrideTemplate)) {
   const DirectorOverrideVars = "%s"
 
   globals.directorWarnedOnceForServiceWithoutHost = false;
@@ -554,14 +554,12 @@ if (! globals[DirectorOverrideTemplate]) {
      * Seems that host is missing when used in a service object, works fine for
      * apply rules
      */
-    try {
-      if (! host) {
-        var host = get_host(host_name)
-      }
-      if (! host) {
-        globals.directorWarnOnceForServiceWithoutHost()
-      }
-    } except { globals.directorWarnOnceForServiceWithoutHost() }
+    if (! host) {
+      var host = get_host(host_name)
+    }
+    if (! host) {
+      globals.directorWarnOnceForServiceWithoutHost()
+    }
 
     if (vars) {
       vars += host.vars[DirectorOverrideVars][name]
@@ -650,7 +648,7 @@ apply Service for (title => params in host.vars["%s"]) {
         $result = $this->db->fetchRow($query);
 
         if (empty($result)) {
-            throw new NotFoundError('Got no config for %s', Util::binary2hex($checksum));
+            throw new NotFoundError('Got no config for %s', bin2hex($checksum));
         }
 
         $this->checksum = $this->binFromDb($result->checksum);
@@ -787,7 +785,7 @@ apply Service for (title => params in host.vars["%s"]) {
 
     public function getLastActivityHexChecksum()
     {
-        return Util::binary2hex($this->getLastActivityChecksum());
+        return bin2hex($this->getLastActivityChecksum());
     }
 
     /**
