@@ -121,6 +121,12 @@ class IcingaCommandArgument extends IcingaObject
 
         unset($plain->id);
         unset($plain->argument_name);
+        if (! isset($plain->argument_value)) {
+            unset($plain->argument_format);
+        }
+        if (! isset($plain->set_if)) {
+            unset($plain->set_if_format);
+        }
 
         $this->transformPlainArgumentValue($plain);
 
@@ -147,7 +153,6 @@ class IcingaCommandArgument extends IcingaObject
         );
 
         unset($plain->id);
-        unset($plain->argument_format);
 
         return $plain;
     }
@@ -156,7 +161,8 @@ class IcingaCommandArgument extends IcingaObject
         $resolved = false,
         $skipDefaults = false,
         array $chosenProperties = null,
-        $resolveIds = true
+        $resolveIds = true,
+        $keepId = false
     ) {
         if ($resolved) {
             throw new RuntimeException(
@@ -170,6 +176,12 @@ class IcingaCommandArgument extends IcingaObject
             );
         }
 
+        if ($keepId) {
+            throw new RuntimeException(
+                'IcingaCommandArgument does not support $keepId'
+            );
+        }
+
         // $resolveIds is misused here
         if ($resolveIds) {
             return $this->toCompatPlainObject();
