@@ -853,6 +853,34 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
         return $this->groups->hasBeenModified();
     }
+
+    public function getAppliedGroups()
+    {
+        $this->assertGroupsSupport();
+        if (! $this instanceof IcingaHost) {
+            throw new RuntimeException('getAppliedGroups is only available for hosts currently!');
+        }
+
+        $type = strtolower($this->getType());
+        $query = $this->db->select()->from(
+            ['gr' => "icinga_${type}group_${type}_resolved"],
+            ['g.object_name']
+        )->join(
+            ['g' => "icinga_${type}group"],
+            "g.id = gr.${type}group_id",
+            []
+        )->joinLeft(
+            ['go' => "icinga_${type}group_${type}"],
+            "go.${type}group_id = gr.${type}group_id AND go.${type}_id = " . $this->id,
+            []
+        )->where(
+            "gr.${type}_id = ?",
+            $this->id
+        )->where("go.${type}_id IS NULL")->order('g.object_name');
+
+        return $this->db->fetchCol($query);
+    }
+
     /**
      * @return IcingaTimePeriodRanges
      */
@@ -888,7 +916,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     {
         $this->assertImportsSupport();
         if ($this->imports === null) {
-            if ($this->hasBeenLoadedFromDb()) {
+            // can not use hasBeenLoadedFromDb() when in onStore()
+            if ($this->getProperty('id') !== null) {
                 $this->imports = IcingaObjectImports::loadForStoredObject($this);
             } else {
                 $this->imports = new IcingaObjectImports($this);
@@ -1057,16 +1086,17 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
     public function getVars()
     {
-        $vars = (object) array();
+        $vars = [];
         foreach ($this->vars() as $key => $var) {
             if ($var->hasBeenDeleted()) {
                 continue;
             }
 
-            $vars->$key = $var->getValue();
+            $vars[$key] = $var->getValue();
         }
+        ksort($vars);
 
-        return $vars;
+        return (object) $vars;
     }
 
     /**
@@ -1211,6 +1241,9 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             throw new RuntimeException($e->getMessage(), 0, $e);
         }
 
+        // Eventually trigger loop detection
+        $this->listAncestorIds();
+
         foreach ($imports as $object) {
             $v = $object->getSingleResolvedProperty($key);
             if (null !== $v) {
@@ -1753,6 +1786,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
                 if ($zoneId = $this->getSingleResolvedProperty('zone_id')) {
                     return $this->getNameForZoneId($zoneId, $config);
                 }
+            } catch (NestingError $e) {
+                throw $e;
             } catch (Exception $e) {
                 return self::RESOLVE_ERROR;
             }
@@ -1819,13 +1854,17 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
     /**
      * @codingStandardsIgnoreStart
      */
-    protected function renderLegacyHost_id()
+    protected function renderLegacyHost_id($value)
     {
-        return $this->renderLegacyRelationProperty(
-            'host',
-            $this->get('host_id'),
-            'host_name'
-        );
+        if (is_array($value)) {
+            return c1::renderKeyValue('host_name', c1::renderArray($value));
+        } else {
+            return $this->renderLegacyRelationProperty(
+                'host',
+                $this->get('host_id'),
+                'host_name'
+            );
+        }
     }
 
     /**
@@ -2043,6 +2082,13 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             }
         }
 
+        if ($this->propertyIsInterval($key)) {
+            return c1::renderKeyValue(
+                $this->intervalProperties[$key],
+                c1::renderInterval($value)
+            );
+        }
+
         if (substr($key, -3) === '_id'
              && $this->hasRelation($relKey = substr($key, 0, -3))
         ) {
@@ -2113,8 +2159,12 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      */
     protected function renderLegacyGroups()
     {
-        if ($this->supportsGroups()) {
-            return $this->groups()->toLegacyConfigString();
+        if ($this->supportsGroups() && $this->hasBeenLoadedFromDb()) {
+            $applied = array();
+            if ($this instanceof IcingaHost) {
+                $applied = $this->getAppliedGroups();
+            }
+            return $this->groups()->toLegacyConfigString($applied);
         } else {
             return '';
         }
@@ -2133,6 +2183,19 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         return $out;
     }
 
+    /**
+     * @return string
+     */
+    protected function renderLegacyMultiRelations()
+    {
+        $out = '';
+        foreach ($this->loadAllMultiRelations() as $rel) {
+            $out .= $rel->toLegacyConfigString();
+        }
+
+        return $out;
+    }
+
     /**
      * @return string
      */
@@ -2323,23 +2386,38 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      */
     public function renderAssign_Filter()
     {
-        // @codingStandardsIgnoreEnd
         return '    ' . AssignRenderer::forFilter(
             Filter::fromQueryString($this->get('assign_filter'))
         )->renderAssign() . "\n";
     }
 
+    public function renderLegacyAssign_Filter()
+    {
+        // @codingStandardsIgnoreEnd
+        if ($this instanceof IcingaHostGroup) {
+            $c = "    # resolved memberships are set via the individual object\n";
+        } elseif ($this instanceof IcingaService) {
+            $c = "    # resolved objects are listed here\n";
+        } else {
+            $c = "    # assign is not supported for " . $this->type . "\n";
+        }
+        $c .= '    #' . AssignRenderer::forFilter(
+            Filter::fromQueryString($this->get('assign_filter'))
+        )->renderAssign() . "\n";
+        return $c;
+    }
+
     public function toLegacyConfigString()
     {
         $str = implode(array(
             $this->renderLegacyObjectHeader(),
             $this->renderLegacyImports(),
             $this->renderLegacyProperties(),
-            $this->renderLegacyRanges(),
             //$this->renderArguments(),
             //$this->renderRelatedSets(),
             $this->renderLegacyGroups(),
-            //$this->renderMultiRelations(),
+            $this->renderLegacyMultiRelations(),
+            $this->renderLegacyRanges(),
             $this->renderLegacyCustomExtensions(),
             $this->renderLegacyCustomVars(),
             $this->renderLegacySuffix()
@@ -2384,11 +2462,11 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
             $this->renderPrioritizedProperties(),
             $this->renderImports(),
             $this->renderProperties(),
-            $this->renderRanges(),
             $this->renderArguments(),
             $this->renderRelatedSets(),
             $this->renderGroups(),
             $this->renderMultiRelations(),
+            $this->renderRanges(),
             $this->renderCustomExtensions(),
             $this->renderCustomVars(),
             $this->renderSuffix()
@@ -2396,7 +2474,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
 
         if ($this->isDisabled()) {
             return "/* --- This object has been disabled ---\n"
-                . $str . "*/\n";
+                // Do not allow strings to break our comment
+                . str_replace('*/', "* /", $str) . "*/\n";
         } else {
             return $str;
         }
@@ -2694,6 +2773,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
      * @param bool $skipDefaults
      * @param array|null $chosenProperties
      * @param bool $resolveIds
+     * @param bool $keepId
      * @return object
      * @throws NotFoundError
      */
@@ -2701,7 +2781,8 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         $resolved = false,
         $skipDefaults = false,
         array $chosenProperties = null,
-        $resolveIds = true
+        $resolveIds = true,
+        $keepId = false
     ) {
         $props = array();
 
@@ -2720,7 +2801,7 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         foreach ($p as $k => $v) {
             // Do not ship ids for IcingaObjects:
             if ($resolveIds) {
-                if ($k === 'id' && $this->hasProperty('object_name')) {
+                if ($k === 'id' && $keepId === false && $this->hasProperty('object_name')) {
                     continue;
                 }
 
@@ -2873,6 +2954,11 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
     }
 
+    public function listFlatResolvedImportNames()
+    {
+        return $this->templateTree()->getAncestorsFor($this);
+    }
+
     public function listImportIds()
     {
         return $this->templateTree()->listParentIdsFor($this);
@@ -2902,6 +2988,27 @@ abstract class IcingaObject extends DbObject implements IcingaConfigRenderer
         }
     }
 
+    protected function mapHostsToZones($names)
+    {
+        $map = array();
+
+        foreach ($names as $hostname) {
+            /** @var IcingaHost $host */
+            $host = IcingaHost::load($hostname, $this->connection);
+
+            $zone = $host->getRenderingZone();
+            if (! array_key_exists($zone, $map)) {
+                $map[$zone] = array();
+            }
+
+            $map[$zone][] = $hostname;
+        }
+
+        ksort($map);
+
+        return $map;
+    }
+
     public function getUrlParams()
     {
         $params = array();
