@@ -7,6 +7,8 @@ use Icinga\Exception\IcingaException;
 use Icinga\Module\Director\Data\PropertiesFilter;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Db\Cache\PrefetchCache;
+use Icinga\Module\Director\DirectorObject\Automation\ExportInterface;
+use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
 use Icinga\Module\Director\IcingaConfig\IcingaConfigHelper as c;
 use Icinga\Module\Director\IcingaConfig\IcingaLegacyConfigHelper as c1;
@@ -15,7 +17,7 @@ use Icinga\Module\Director\Resolver\HostServiceBlacklist;
 use InvalidArgumentException;
 use RuntimeException;
 
-class IcingaService extends IcingaObject
+class IcingaService extends IcingaObject implements ExportInterface
 {
     use FlappingSupport;
 
@@ -115,6 +117,9 @@ class IcingaService extends IcingaObject
         'apply_for'
     ];
 
+    /** @var ServiceGroupMembershipResolver */
+    protected $servicegroupMembershipResolver;
+
     /**
      * @return IcingaCommand
      * @throws IcingaException
@@ -150,6 +155,100 @@ class IcingaService extends IcingaObject
         return $this->get('use_var_overrides') === 'y';
     }
 
+    public function getUniqueIdentifier()
+    {
+        if ($this->isTemplate()) {
+            return $this->getObjectName();
+        } else {
+            throw new RuntimeException(
+                'getUniqueIdentifier() is supported by Service Templates only'
+            );
+        }
+    }
+
+    /**
+     * @return object
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public function export()
+    {
+        // TODO: ksort in toPlainObject?
+        $props = (array) $this->toPlainObject();
+        $props['fields'] = $this->loadFieldReferences();
+        ksort($props);
+
+        return (object) $props;
+    }
+
+    /**
+     * @param $plain
+     * @param Db $db
+     * @param bool $replace
+     * @return IcingaService
+     * @throws DuplicateKeyException
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public static function import($plain, Db $db, $replace = false)
+    {
+        $properties = (array) $plain;
+        $name = $properties['object_name'];
+        if ($properties['object_type'] !== 'template') {
+            throw new InvalidArgumentException(sprintf(
+                'Can import only Templates, got "%s" for "%s"',
+                $properties['object_type'],
+                $name
+            ));
+        }
+        $key = [
+            'object_type' => 'template',
+            'object_name' => $name
+        ];
+
+        if ($replace && static::exists($key, $db)) {
+            $object = static::load($key, $db);
+        } elseif (static::exists($key, $db)) {
+            throw new DuplicateKeyException(
+                'Service Template "%s" already exists',
+                $name
+            );
+        } else {
+            $object = static::create([], $db);
+        }
+
+        // $object->newFields = $properties['fields'];
+        unset($properties['fields']);
+        $object->setProperties($properties);
+
+        return $object;
+    }
+
+    protected function loadFieldReferences()
+    {
+        $db = $this->getDb();
+
+        $res = $db->fetchAll(
+            $db->select()->from([
+                'sf' => 'icinga_service_field'
+            ], [
+                'sf.datafield_id',
+                'sf.is_required',
+                'sf.var_filter',
+            ])->join(['df' => 'director_datafield'], 'df.id = sf.datafield_id', [])
+                ->where('service_id = ?', $this->get('id'))
+                ->order('varname ASC')
+        );
+
+        if (empty($res)) {
+            return [];
+        } else {
+            foreach ($res as $field) {
+                $field->datafield_id = (int) $field->datafield_id;
+            }
+
+            return $res;
+        }
+    }
+
     /**
      * @param string $key
      * @return $this
@@ -197,8 +296,6 @@ class IcingaService extends IcingaObject
      */
     public function renderHost_id()
     {
-        // @codingStandardsIgnoreEnd
-
         if ($this->hasBeenAssignedToHostTemplate()) {
             return '';
         }
@@ -206,6 +303,28 @@ class IcingaService extends IcingaObject
         return $this->renderRelationProperty('host', $this->get('host_id'), 'host_name');
     }
 
+    /**
+     * @codingStandardsIgnoreStart
+     */
+    protected function renderLegacyHost_id($value)
+    {
+        // @codingStandardsIgnoreEnd
+        if (is_array($value)) {
+            $blacklisted = $this->getBlacklistedHostnames();
+            $c = c1::renderKeyValue('host_name', c1::renderArray(array_diff($value, $blacklisted)));
+
+            // blacklisted in this (zoned) scope?
+            $bl = array_intersect($blacklisted, $value);
+            if (! empty($bl)) {
+                $c .= c1::renderKeyValue('# ignored on', c1::renderArray($bl));
+            }
+
+            return $c;
+        } else {
+            return parent::renderLegacyHost_id($value);
+        }
+    }
+
     /**
      * @param IcingaConfig $config
      * @throws IcingaException
@@ -231,37 +350,22 @@ class IcingaService extends IcingaObject
 
         $assign_filter = $this->get('assign_filter');
         $filter = Filter::fromQueryString($assign_filter);
-        $hosts = HostApplyMatches::forFilter($filter, $conn);
+        $hostnames = HostApplyMatches::forFilter($filter, $conn);
+
         $this->set('object_type', 'object');
-        $this->set('assign_filter', null);
 
-        foreach ($hosts as $hostname) {
-            $file = $this->legacyHostnameServicesFile($hostname, $config);
-            $this->set('host', $hostname);
-            $file->addLegacyObject($this);
+        foreach ($this->mapHostsToZones($hostnames) as $zone => $names) {
+            $this->set('host_id', $names);
+
+            $blacklisted = $this->getBlacklistedHostnames();
+            $zoneNames = array_diff($names, $blacklisted);
+            if (empty($zoneNames)) {
+                continue;
+            }
+
+            $config->configFile('director/' . $zone . '/service_apply', '.cfg')
+                ->addLegacyObject($this);
         }
-
-        $this->set('host', null);
-        $this->set('object_type', 'apply');
-        $this->set('assign_filter', $assign_filter);
-    }
-
-    /**
-     * @param string $hostname
-     * @param IcingaConfig $config
-     * @return \Icinga\Module\Director\IcingaConfig\IcingaConfigFile
-     * @throws \Icinga\Exception\NotFoundError
-     */
-    protected function legacyHostnameServicesFile($hostname, IcingaConfig $config)
-    {
-        return $config->configFile(
-            sprintf(
-                'director/%s/service_apply',
-                IcingaHost::load($hostname, $this->getConnection())
-                    ->getRenderingZone($config)
-            ),
-            '.cfg'
-        );
     }
 
     /**
@@ -273,10 +377,6 @@ class IcingaService extends IcingaObject
             return '';
         }
 
-        if ($this->isApplyRule()) {
-            throw new InvalidArgumentException('Apply Services can not be rendered directly.');
-        }
-
         $str = parent::toLegacyConfigString();
 
         if (! $this->isDisabled()
@@ -305,7 +405,9 @@ class IcingaService extends IcingaObject
             && $this->get('host_id')
             && $this->getRelated('host')->isDisabled()
         ) {
-            return "/* --- This services host has been disabled ---\n$str*/\n";
+            return "/* --- This services host has been disabled ---\n"
+                // Do not allow strings to break our comment
+                . str_replace('*/', "* /", $str) . "*/\n";
         } else {
             return $str;
         }
@@ -395,6 +497,7 @@ class IcingaService extends IcingaObject
 
     /**
      * @return string
+     * @throws \Icinga\Exception\NotFoundError
      */
     protected function renderCustomExtensions()
     {
@@ -411,16 +514,31 @@ class IcingaService extends IcingaObject
         }
 
         $blacklist = $this->getBlacklistedHostnames();
-        if (! empty($blacklist)) {
-            if (count($blacklist) === 1) {
+        $blacklistedTemplates = [];
+        $blacklistedHosts = [];
+        foreach ($blacklist as $hostname) {
+            if (IcingaHost::load($hostname, $this->connection)->isTemplate()) {
+                $blacklistedTemplates[] = $hostname;
+            } else {
+                $blacklistedHosts[] = $hostname;
+            }
+        }
+        foreach ($blacklistedTemplates as $template) {
+            $output .= sprintf(
+                "    ignore where %s in host.templates\n",
+                c::renderString($template)
+            );
+        }
+        if (! empty($blacklistedHosts)) {
+            if (count($blacklistedHosts) === 1) {
                 $output .= sprintf(
                     "    ignore where host.name == %s\n",
-                    c::renderString($blacklist[0])
+                    c::renderString($blacklistedHosts[0])
                 );
             } else {
                 $output .= sprintf(
                     "    ignore where host.name in %s\n",
-                    c::renderArray($blacklist)
+                    c::renderArray($blacklistedHosts)
                 );
             }
         }
@@ -494,6 +612,8 @@ class IcingaService extends IcingaObject
     {
         if ($this->get('host_id')) {
             return 'director/host/services?name=' . rawurlencode($this->get('host'));
+        } elseif ($this->get('service_set_id')) {
+            return 'director/serviceset/services?name=' . rawurlencode($this->get('service_set'));
         } else {
             return parent::getOnDeleteUrl();
         }
@@ -633,4 +753,30 @@ class IcingaService extends IcingaObject
             );
         }
     }
+
+    protected function notifyResolvers()
+    {
+        $resolver = $this->getServiceGroupMembershipResolver();
+        $resolver->addObject($this);
+        $resolver->refreshDb();
+
+        return $this;
+    }
+
+    protected function getServiceGroupMembershipResolver()
+    {
+        if ($this->servicegroupMembershipResolver === null) {
+            $this->servicegroupMembershipResolver = new ServiceGroupMembershipResolver(
+                $this->getConnection()
+            );
+        }
+
+        return $this->servicegroupMembershipResolver;
+    }
+
+    public function setServiceGroupMembershipResolver(ServiceGroupMembershipResolver $resolver)
+    {
+        $this->servicegroupMembershipResolver = $resolver;
+        return $this;
+    }
 }
