@@ -2,12 +2,18 @@
 
 namespace Icinga\Module\Director\Objects;
 
+use Exception;
 use Icinga\Data\Filter\Filter;
+use Icinga\Module\Director\Db;
+use Icinga\Module\Director\Db\Cache\PrefetchCache;
+use Icinga\Module\Director\DirectorObject\Automation\ExportInterface;
 use Icinga\Module\Director\Exception\DuplicateKeyException;
 use Icinga\Module\Director\IcingaConfig\IcingaConfig;
+use Icinga\Module\Director\Resolver\HostServiceBlacklist;
 use InvalidArgumentException;
+use RuntimeException;
 
-class IcingaServiceSet extends IcingaObject
+class IcingaServiceSet extends IcingaObject implements ExportInterface
 {
     protected $table = 'icinga_service_set';
 
@@ -47,7 +53,7 @@ class IcingaServiceSet extends IcingaObject
     protected function setKey($key)
     {
         if (is_int($key)) {
-            $this->id = $key;
+            $this->set('id', $key);
         } elseif (is_string($key)) {
             $keyComponents = preg_split('~!~', $key);
             if (count($keyComponents) === 1) {
@@ -68,6 +74,7 @@ class IcingaServiceSet extends IcingaObject
 
     /**
      * @return IcingaService[]
+     * @throws \Icinga\Exception\NotFoundError
      */
     public function getServiceObjects()
     {
@@ -82,6 +89,11 @@ class IcingaServiceSet extends IcingaObject
         }
     }
 
+    /**
+     * @param IcingaServiceSet $set
+     * @return array
+     * @throws \Icinga\Exception\NotFoundError
+     */
     protected function getServiceObjectsForSet(IcingaServiceSet $set)
     {
         if ($set->get('id') === null) {
@@ -109,6 +121,122 @@ class IcingaServiceSet extends IcingaObject
         return $services;
     }
 
+    public function getUniqueIdentifier()
+    {
+        return $this->getObjectName();
+    }
+
+    /**
+     * @return object
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public function export()
+    {
+        if ($this->get('host_id')) {
+            return $this->exportSetOnHost();
+        } else {
+            return $this->exportTemplate();
+        }
+    }
+
+    protected function exportSetOnHost()
+    {
+        // TODO.
+        throw new RuntimeException('Not yet');
+    }
+
+    /**
+     * @return object
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    protected function exportTemplate()
+    {
+        $props = $this->getProperties();
+        unset($props['id'], $props['host_id']);
+        $props['services'] = [];
+        foreach ($this->getServiceObjects() as $serviceObject) {
+            $props['services'][$serviceObject->getObjectName()] = $serviceObject->export();
+        }
+        ksort($props);
+
+        return (object) $props;
+    }
+
+    /**
+     * @param $plain
+     * @param Db $db
+     * @param bool $replace
+     * @return IcingaServiceSet
+     * @throws DuplicateKeyException
+     * @throws \Icinga\Exception\NotFoundError
+     */
+    public static function import($plain, Db $db, $replace = false)
+    {
+        $properties = (array) $plain;
+        $name = $properties['object_name'];
+        if (isset($properties['services'])) {
+            $services = $properties['services'];
+            unset($properties['services']);
+        } else {
+            $services = [];
+        }
+
+        if ($properties['object_type'] !== 'template') {
+            throw new InvalidArgumentException(sprintf(
+                'Can import only Templates, got "%s" for "%s"',
+                $properties['object_type'],
+                $name
+            ));
+        }
+        if ($replace && static::exists($name, $db)) {
+            $object = static::load($name, $db);
+        } elseif (static::exists($name, $db)) {
+            throw new DuplicateKeyException(
+                'Service Set "%s" already exists',
+                $name
+            );
+        } else {
+            $object = static::create([], $db);
+        }
+
+        $object->setProperties($properties);
+
+        // This is not how other imports work, but here we need an ID
+        if (! $object->hasBeenLoadedFromDb()) {
+            $object->store();
+        }
+
+        $setId = $object->get('id');
+        $sQuery = $db->getDbAdapter()->select()->from(
+            ['s' => 'icinga_service'],
+            's.*'
+        )->where('service_set_id = ?', $setId);
+        $existingServices = IcingaService::loadAll($db, $sQuery, 'object_name');
+        foreach ($services as $service) {
+            if (isset($service->fields)) {
+                unset($service->fields);
+            }
+            $name = $service->object_name;
+            if (isset($existingServices[$name])) {
+                $existing = $existingServices[$name];
+                $existing->setProperties((array) $service);
+                $existing->set('service_set_id', $setId);
+                if ($existing->hasBeenModified()) {
+                    $existing->store();
+                }
+            } else {
+                $new = IcingaService::create((array) $service, $db);
+                $new->set('service_set_id', $setId);
+                $new->store();
+            }
+        }
+
+        return $object;
+    }
+
+    /**
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function onDelete()
     {
         $hostId = $this->get('host_id');
@@ -133,6 +261,10 @@ class IcingaServiceSet extends IcingaObject
         parent::onDelete();
     }
 
+    /**
+     * @param IcingaConfig $config
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function renderToConfig(IcingaConfig $config)
     {
         if ($this->get('assign_filter') === null && $this->isTemplate()) {
@@ -159,9 +291,13 @@ class IcingaServiceSet extends IcingaObject
                 $service->set('object_type', 'apply');
                 $service->set('assign_filter', $filter);
             } elseif ($hostId = $this->get('host_id')) {
+                $host = $this->getRelatedObject('host', $hostId)->getObjectName();
+                if (in_array($host, $this->getBlacklistedHostnames($service))) {
+                    continue;
+                }
                 $service->set('object_type', 'object');
                 $service->set('use_var_overrides', 'y');
-                $service->set('host_id', $this->get('host_id'));
+                $service->set('host_id', $hostId);
             } else {
                 // Service set template without assign filter or host
                 continue;
@@ -172,6 +308,23 @@ class IcingaServiceSet extends IcingaObject
         }
     }
 
+    /**
+     * @return array
+     */
+    public function getBlacklistedHostnames($service)
+    {
+        // Hint: if ($this->isApplyRule()) would be nice, but apply rules are
+        // not enough, one might want to blacklist single services from Sets
+        // assigned to single Hosts.
+        if (PrefetchCache::shouldBeUsed()) {
+            $lookup = PrefetchCache::instance()->hostServiceBlacklist();
+        } else {
+            $lookup = new HostServiceBlacklist($this->getConnection());
+        }
+
+        return $lookup->getBlacklistedHostnamesForService($service);
+    }
+
     protected function getConfigFileWithHeader(IcingaConfig $config)
     {
         $file = $config->configFile(
@@ -208,6 +361,10 @@ class IcingaServiceSet extends IcingaObject
         return $this;
     }
 
+    /**
+     * @param IcingaConfig $config
+     * @throws \Icinga\Exception\NotFoundError
+     */
     public function renderToLegacyConfig(IcingaConfig $config)
     {
         if ($this->get('assign_filter') === null && $this->isTemplate()) {
@@ -223,49 +380,41 @@ class IcingaServiceSet extends IcingaObject
         // Delegating this to the service would look, but this way it's faster
         if ($filter = $this->get('assign_filter')) {
             $filter = Filter::fromQueryString($filter);
-            $hosts = HostApplyMatches::forFilter($filter, $conn);
+
+            $hostnames = HostApplyMatches::forFilter($filter, $conn);
+        } else {
+            $hostnames = array($this->getRelated('host')->getObjectName());
+        }
+
+        $blacklists = [];
+
+        foreach ($this->mapHostsToZones($hostnames) as $zone => $names) {
+            $file = $config->configFile('director/' . $zone . '/servicesets', '.cfg');
+            $file->addContent($this->getConfigHeaderComment($config));
+
             foreach ($this->getServiceObjects() as $service) {
+                $object_name = $service->getObjectName();
+
+                if (! array_key_exists($object_name, $blacklists)) {
+                    $blacklists[$object_name] = $service->getBlacklistedHostnames();
+                }
+
+                // check if all hosts in the zone ignore this service
+                $zoneNames = array_diff($names, $blacklists[$object_name]);
+                if (empty($zoneNames)) {
+                    continue;
+                }
+
                 $service->set('object_type', 'object');
+                $service->set('host_id', $names);
+
                 $this->copyVarsToService($service);
 
-                foreach ($hosts as $hostname) {
-                    $file = $this->legacyHostnameServicesFile($hostname, $config);
-                    $file->addContent($this->getConfigHeaderComment($config));
-                    $service->set('host', $hostname);
-                    $file->addLegacyObject($service);
-                }
-            }
-        } else {
-            foreach ($this->getServiceObjects() as $service) {
-                $service->set('object_type', 'object');
-                $service->set('host_id', $this->get('host_id'));
-                foreach ($this->vars() as $k => $var) {
-                    $service->$k = $var;
-                }
-                $file = $this->legacyRelatedHostFile($service, $config);
-                $file->addContent($this->getConfigHeaderComment($config));
                 $file->addLegacyObject($service);
             }
         }
     }
 
-    protected function legacyHostnameServicesFile($hostname, IcingaConfig $config)
-    {
-        $host = IcingaHost::load($hostname, $this->getConnection());
-        return $config->configFile(
-            'director/' . $host->getRenderingZone($config) . '/servicesets',
-            '.cfg'
-        );
-    }
-
-    protected function legacyRelatedHostFile(IcingaService $service, IcingaConfig $config)
-    {
-        return $config->configFile(
-            'director/' . $service->getRelated('host')->getRenderingZone($config) . '/servicesets',
-            '.cfg'
-        );
-    }
-
     public function getRenderingZone(IcingaConfig $config = null)
     {
         if ($this->get('host_id') === null) {
@@ -306,6 +455,37 @@ class IcingaServiceSet extends IcingaObject
         return $services;
     }
 
+    /**
+     * Fetch IcingaServiceSet that are based on this set and added to hosts directly
+     *
+     * @return IcingaServiceSet[]
+     */
+    public function fetchHostSets()
+    {
+        $id = $this->get('id');
+        if ($id === null) {
+            return [];
+        }
+
+        $query = $this->db->select()
+            ->from(
+                ['o' => $this->table]
+            )->join(
+                ['ssi' => $this->table . '_inheritance'],
+                'ssi.service_set_id = o.id',
+                []
+            )->where(
+                'ssi.parent_service_set_id = ?',
+                $id
+            );
+
+        return static::loadAll($this->connection, $query);
+    }
+
+    /**
+     * @throws DuplicateKeyException
+     * @throws \Icinga\Exception\NotFoundError
+     */
     protected function beforeStore()
     {
         parent::beforeStore();
@@ -327,4 +507,24 @@ class IcingaServiceSet extends IcingaObject
             );
         }
     }
+
+    public function toSingleIcingaConfig()
+    {
+        $config = parent::toSingleIcingaConfig();
+
+        try {
+            foreach ($this->fetchHostSets() as $set) {
+                $set->renderToConfig($config);
+            }
+        } catch (Exception $e) {
+            $config->configFile(
+                'failed-to-render'
+            )->prepend(
+                "/** Failed to render this object **/\n"
+                . '/*  ' . $e->getMessage() . ' */'
+            );
+        }
+
+        return $config;
+    }
 }
