@@ -2,10 +2,10 @@
 
 namespace Icinga\Module\Director\Objects;
 
-use Icinga\Exception\ConfigurationError;
 use Icinga\Module\Director\Data\Db\DbObjectWithSettings;
 use Icinga\Module\Director\Hook\PropertyModifierHook;
 use Icinga\Module\Director\Objects\Extension\PriorityColumn;
+use RuntimeException;
 
 class ImportRowModifier extends DbObjectWithSettings
 {
@@ -39,7 +39,10 @@ class ImportRowModifier extends DbObjectWithSettings
             $class = $this->get('provider_class');
             /** @var PropertyModifierHook $obj */
             if (! class_exists($class)) {
-                throw new ConfigurationError('Cannot instantiate Property modifier %s', $class);
+                throw new RuntimeException(sprintf(
+                    'Cannot instantiate Property modifier %s',
+                    $class
+                ));
             }
             $obj = new $class;
             $obj->setSettings($this->getSettings());
@@ -60,6 +63,7 @@ class ImportRowModifier extends DbObjectWithSettings
         unset($properties['id']);
         unset($properties['source_id']);
         $properties['settings'] = (object) $this->getSettings();
+        ksort($properties);
 
         return (object) $properties;
     }
