@@ -6,6 +6,7 @@ use Exception;
 use Icinga\Application\Config;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\Objects\IcingaHost;
+use Icinga\Module\Director\Util;
 use Icinga\Module\Monitoring\Hook\HostActionsHook;
 use Icinga\Module\Monitoring\Object\Host;
 use Icinga\Web\Url;
@@ -23,25 +24,27 @@ class HostActions extends HostActionsHook
 
     protected function getThem(Host $host)
     {
+        $actions = array();
         $db = $this->db();
         if (! $db) {
-            return array();
+            return $actions;
+        }
+        $hostname = $host->host_name;
+        if (Util::hasPermission('director/inspect')) {
+            $actions['Inspect'] = Url::fromPath(
+                'director/inspect/object',
+                array('type' => 'host', 'plural' => 'hosts', 'name' => $hostname)
+            );
         }
 
-        if (IcingaHost::exists($host->host_name, $db)) {
-            return array(
-                'Modify' => Url::fromPath(
-                    'director/host/edit',
-                    array('name' => $host->host_name)
-                ),
-                'Inspect' => Url::fromPath(
-                    'director/inspect/object',
-                    array('type' => 'host', 'plural' => 'hosts', 'name' => $host->host_name)
-                )
+        if (Util::hasPermission('director/hosts') && IcingaHost::exists($hostname, $db)) {
+            $actions['Modify'] = Url::fromPath(
+                'director/host/edit',
+                array('name' => $hostname)
             );
-        } else {
-            return array();
         }
+
+        return $actions;
     }
 
     protected function db()
