@@ -2,6 +2,9 @@
 
 namespace Icinga\Module\Director\Web\Controller;
 
+use Icinga\Module\Director\DirectorObject\Automation\ExportInterface;
+use Icinga\Module\Director\Exception\NestingError;
+use Icinga\Module\Director\Objects\IcingaCommand;
 use Icinga\Module\Director\Objects\IcingaObject;
 use Icinga\Module\Director\Web\Controller\Extension\DirectorDb;
 use Icinga\Module\Director\Web\Table\ApplyRulesTable;
@@ -99,6 +102,9 @@ abstract class TemplateController extends CompatController
     public function usageAction()
     {
         $template = $this->requireTemplate();
+        if (! $template->isTemplate() && $template instanceof IcingaCommand) {
+            $this->redirectNow($this->url()->setPath('director/command'));
+        }
         $templateName = $template->getObjectName();
 
         $type = $this->getType();
@@ -115,6 +121,18 @@ abstract class TemplateController extends CompatController
                 ['class' => 'icon-edit']
             )
         ]);
+        if ($template instanceof ExportInterface) {
+            $this->actions()->add(Link::create(
+                $this->translate('Add to Basket'),
+                'director/basket/add',
+                [
+                    'type'  => ucfirst($this->getType()) . 'Template',
+                    'names' => $template->getUniqueIdentifier()
+                ],
+                ['class' => 'icon-tag']
+            ));
+        }
+
         $list = new UnorderedList([], [
             'class' => 'vertical-action-list'
         ]);
@@ -165,9 +183,13 @@ abstract class TemplateController extends CompatController
             Html::tag('h2', null, $this->translate('Current Template Usage'))
         );
 
-        $this->content()->add(
-            TemplateUsageTable::forTemplate($template)
-        );
+        try {
+            $this->content()->add(
+                TemplateUsageTable::forTemplate($template)
+            );
+        } catch (NestingError $e) {
+            $this->content()->add(Html::tag('p', ['class' => 'error'], $e->getMessage()));
+        }
     }
 
     protected function getType()
