@@ -4,9 +4,11 @@ namespace Icinga\Module\Director\Web\Form;
 
 use Icinga\Application\Icinga;
 use Icinga\Application\Modules\Module;
+use dipl\Html\Html;
+use dipl\Html\ValidHtml;
 use Zend_Form;
 
-abstract class QuickBaseForm extends Zend_Form
+abstract class QuickBaseForm extends Zend_Form implements ValidHtml
 {
     /**
      * The Icinga module this form belongs to. Usually only set if the
@@ -58,13 +60,11 @@ abstract class QuickBaseForm extends Zend_Form
             ucfirst($module->getName())
         );
 
-        $this->addPrefixPaths(array(
-            array(
-                'prefix'    => __NAMESPACE__ . '\\Element\\',
-                'path'      => $basedir . '/Element',
-                'type'      => static::ELEMENT
-            )
-        ));
+        $this->addPrefixPath(
+            __NAMESPACE__ . '\\Element\\',
+            $basedir . '/Element',
+            static::ELEMENT
+        );
 
         return $this;
     }
@@ -83,13 +83,20 @@ abstract class QuickBaseForm extends Zend_Form
     }
 
     // TODO: Should be an element
-    public function addHtmlHint($html, $options = array())
+    public function addHtmlHint($html, $options = [])
     {
-        return $this->addHtml('<div class="hint">' . $html . '</div>', $options);
+        return $this->addHtml(
+            Html::tag('div', ['class' => 'hint'], $html),
+            $options
+        );
     }
 
-    public function addHtml($html, $options = array())
+    public function addHtml($html, $options = [])
     {
+        if ($html instanceof ValidHtml) {
+            $html = $html->render();
+        }
+
         if (array_key_exists('name', $options)) {
             $name = $options['name'];
             unset($options['name']);
