@@ -2,6 +2,8 @@
 
 namespace Icinga\Module\Director\Web\Table;
 
+use dipl\Html\BaseHtmlElement;
+use dipl\Html\Html;
 use Icinga\Module\Director\Forms\RemoveLinkForm;
 use Icinga\Module\Director\Objects\IcingaHost;
 use Icinga\Module\Director\Objects\IcingaServiceSet;
@@ -27,6 +29,12 @@ class IcingaServiceSetServiceTable extends ZfQueryBasedTable
         'service',
     ];
 
+    /** @var bool */
+    protected $readonly = false;
+
+    /** @var string|null */
+    protected $highlightedService;
+
     /**
      * @param IcingaServiceSet $set
      * @return static
@@ -69,6 +77,26 @@ class IcingaServiceSetServiceTable extends ZfQueryBasedTable
         return $this;
     }
 
+    /**
+     * Show no related links
+     *
+     * @param bool $readonly
+     * @return $this
+     */
+    public function setReadonly($readonly = true)
+    {
+        $this->readonly = (bool) $readonly;
+
+        return $this;
+    }
+
+    public function highlightService($service)
+    {
+        $this->highlightedService = $service;
+
+        return $this;
+    }
+
     protected function addHeaderColumnsTo(HtmlElement $parent)
     {
         if ($this->host || $this->affectedHost) {
@@ -82,10 +110,18 @@ class IcingaServiceSetServiceTable extends ZfQueryBasedTable
 
     /**
      * @param $row
-     * @return Link
+     * @return BaseHtmlElement
      */
     protected function getServiceLink($row)
     {
+        if ($this->readonly) {
+            if ($this->highlightedService === $row->service) {
+                return Html::tag('span', ['class' => 'icon-right-big'], $row->service);
+            } else {
+                return $row->service;
+            }
+        }
+
         if ($this->affectedHost) {
             $params = [
                 'name'    => $this->affectedHost->getObjectName(),
@@ -141,6 +177,11 @@ class IcingaServiceSetServiceTable extends ZfQueryBasedTable
     {
         if (! $this->host) {
             $deleteLink = '';
+        } elseif ($this->readonly) {
+            $deleteLink = Html::tag('span', [
+                'class' => 'icon-paste',
+                'style' => 'float: right; font-weight: normal',
+            ], $this->host->getObjectName());
         } elseif ($this->affectedHost->get('id') !== $this->host->get('id')) {
             $host = $this->host;
             $deleteLink = Link::create(
@@ -160,19 +201,33 @@ class IcingaServiceSetServiceTable extends ZfQueryBasedTable
         } else {
             $deleteLink = new RemoveLinkForm(
                 $this->translate('Remove'),
-                $this->translate('Remove this set from this host'),
+                sprintf(
+                    $this->translate('Remove "%s" from this host'),
+                    $this->getTitle()
+                ),
                 Url::fromPath('director/host/services', [
                     'name' => $this->host->getObjectName()
-                ])
+                ]),
+                ['title' => $this->getTitle()]
             );
-            $hostId = $this->host->get('id');
-            $setName = $this->set->getObjectName();
-            $db = $this->set->getConnection();
-            $deleteLink->runOnSuccess(function () use ($db, $hostId, $setName) {
-                IcingaServiceSet::load([
-                    'host_id' => $hostId,
-                    'object_name' => $setName
-                ], $db)->delete();
+            $deleteLink->runOnSuccess(function () {
+                $conn = $this->set->getConnection();
+                $db = $conn->getDbAdapter();
+                $query = $db->select()->from(
+                    ['ss' => 'icinga_service_set'],
+                    'ss.id'
+                )->join(
+                    ['ssih' => 'icinga_service_set_inheritance'],
+                    'ssih.service_set_id = ss.id',
+                    []
+                )->where(
+                    'ssih.parent_service_set_id = ?',
+                    $this->set->get('id')
+                )->where('ss.host_id = ?', $this->host->get('id'));
+                IcingaServiceSet::loadWithAutoIncId(
+                    $db->fetchOne($query),
+                    $conn
+                )->delete();
             });
             $deleteLink->handleRequest();
         }
