@@ -15,14 +15,15 @@ use Icinga\Exception\QueryException;
 use Icinga\Module\Director\Db;
 use Icinga\Module\Director\PlainObjectRenderer;
 use Icinga\Web\Request;
-use Icinga\Web\Url;
+use dipl\Web\Url;
 use Icinga\Web\View;
 use Icinga\Web\Widget;
 use Icinga\Web\Widget\Paginator;
+use dipl\Html\ValidHtml;
 use stdClass;
 use Zend_Db_Select as ZfDbSelect;
 
-abstract class QuickTable implements Paginatable
+abstract class QuickTable implements Paginatable, ValidHtml
 {
     protected $view;
 
@@ -133,7 +134,7 @@ abstract class QuickTable implements Paginatable
             if ($value === null) {
                 if ($val === null) {
                     $value = '-';
-                } elseif (is_array($val) || $val instanceof stdClass) {
+                } elseif (is_array($val) || $val instanceof stdClass || is_bool($val)) {
                     $value = '<pre>'
                            . $this->view()->escape(PlainObjectRenderer::render($val))
                            . '</pre>';
@@ -195,7 +196,7 @@ abstract class QuickTable implements Paginatable
         }
         if ($filter) {
             foreach ($enforced as $f) {
-                $filter->andFilter($f);
+                $filter = $filter->andFilter($f);
             }
             $query->where($this->renderFilter($filter));
         }
