@@ -24,6 +24,7 @@
             // Disabled
             // this.module.on('click', 'div.controls ul.tabs a', this.detailTabClick);
             this.module.on('click', 'input.related-action', this.extensibleSetAction);
+            this.module.on('click', 'ul.filter-root input[type=submit]', this.setAutoSubmitted);
             this.module.on('focus', 'form input, form textarea, form select', this.formElementFocus);
             this.module.on('keyup', '.director-suggest', this.autoSuggest);
             this.module.on('keydown', '.director-suggest', this.suggestionKeyDown);
@@ -45,6 +46,9 @@
             }
 
             $a = $(ev.currentTarget);
+            if ($a.hasClass('refresh-container-control')) {
+                return;
+            }
             $container = $a.closest('.container');
             id = $container.attr('id');
 
@@ -274,18 +278,38 @@
          */
         refreshSuggestionList: function($suggestions, $el)
         {
-            $suggestions.load(this.module.icinga.config.baseUrl + '/director/suggest', {
-                value: $el.val(),
-                context: $el.data('suggestion-context'),
-                for_host: $el.data('suggestion-for-host')
-            }, function (responseText, textStatus, jqXHR) {
+            // Not sure whether we need this Accept-header
+            var headers = { 'X-Icinga-Accept': 'text/html' };
+            var icinga = this.module.icinga;
+
+            // Ask for a new window id in case we don't already have one
+            if (icinga.ui.hasWindowId()) {
+                headers['X-Icinga-WindowId'] = icinga.ui.getWindowId();
+            } else {
+                headers['X-Icinga-WindowId'] = 'undefined';
+            }
+
+            var onResponse =  function (data, textStatus, req) {
+                $suggestions.html(data);
                 var $li = $suggestions.find('li');
                 if ($li.length) {
                     $suggestions.show();
                 } else {
                     $suggestions.hide();
                 }
+            };
+
+            var req = $.ajax({
+                type: 'POST',
+                url: this.module.icinga.config.baseUrl + '/director/suggest',
+                data: {
+                    value: $el.val(),
+                    context: $el.data('suggestion-context'),
+                    for_host: $el.data('suggestion-for-host')
+                },
+                headers: headers
             });
+            req.done(onResponse);
 
             return $suggestions;
         },
@@ -577,7 +601,6 @@
             }
 
             var $dd = $input.closest('dd');
-            $dd.find('p.description').show();
             if ($dd.attr('id') && $dd.attr('id').match(/button/)) {
                 return;
             }
@@ -585,19 +608,10 @@
             var $dt = $dd.prev();
             var $form = $dd.closest('form');
 
-            var $desc = $dd.find('p.description');
-            if ($desc.length) {
-                $form.css({ marginBottom: ($desc.height() + 48) + 'px' });
-            }
-
             $form.find('dt, dd, li').removeClass('active');
             $li.addClass('active');
             $dt.addClass('active');
             $dd.addClass('active');
-
-            $form.find('dd').not($dd)
-                .find('p.description')
-                .hide();
         },
 
         highlightFormErrors: function($container)
@@ -620,10 +634,6 @@
             this.openedFieldsets[$fieldset.attr('id')] = ! $fieldset.hasClass('collapsed');
         },
 
-        hideInactiveFormDescriptions: function($container) {
-            $container.find('dd').not('.active').find('p.description').hide();
-        },
-
         beforeRender: function(ev) {
             var $container = $(ev.currentTarget);
             var id = $container.attr('id');
@@ -668,14 +678,24 @@
 
         rendered: function(ev) {
             var iid;
+            var icinga = this.module.icinga;
             var $container = $(ev.currentTarget);
+            if ($container.children('div.controls').first().data('directorWindowId') === '_UNDEFINED_') {
+                var $url = $container.data('icingaUrl');
+                if (typeof $url !== 'undefined') {
+                    icinga.loader.loadUrl($url, $container).autorefresh = true;
+                }
+
+                $container.children('div.controls').children().hide();
+                $container.children('div.content').hide();
+                return;
+            }
             this.restoreContainerFieldsets($container);
             this.backupAllExtensibleSetDefaultValues($container);
             this.highlightFormErrors($container);
             this.scrollHighlightIntoView($container);
             this.scrollActiveRowIntoView($container);
             this.highlightActiveDashlet($container);
-            this.hideInactiveFormDescriptions($container);
             if (iid = $container.data('activeExtensibleEntry')) {
                 $('#' + iid).focus();
                 $container.removeData('activeExtensibleEntry');
