@@ -42,6 +42,7 @@ CREATE TYPE enum_sync_state AS ENUM(
     'failing'
 );
 CREATE TYPE enum_host_service AS ENUM('host', 'service');
+CREATE TYPE enum_owner_type AS ENUM('user', 'usergroup', 'role');
 
 
 CREATE OR REPLACE FUNCTION unix_timestamp(timestamp with time zone) RETURNS bigint AS '
@@ -71,6 +72,46 @@ COMMENT ON COLUMN director_activity_log.old_properties IS 'Property hash, JSON';
 COMMENT ON COLUMN director_activity_log.new_properties IS 'Property hash, JSON';
 
 
+CREATE TABLE director_basket (
+  uuid bytea CHECK(LENGTH(uuid) = 16) NOT NULL,
+  basket_name VARCHAR(64) NOT NULL,
+  owner_type enum_owner_type NOT NULL,
+  owner_value VARCHAR(255) NOT NULL,
+  objects text NOT NULL, -- json-encoded
+  PRIMARY KEY (uuid)
+);
+
+CREATE UNIQUE INDEX basket_basket_name ON director_basket (basket_name);
+
+
+CREATE TABLE director_basket_content (
+  checksum bytea CHECK(LENGTH(checksum) = 20) NOT NULL,
+  summary VARCHAR(500) NOT NULL, -- json
+  content text NOT NULL, -- json
+  PRIMARY KEY (checksum)
+);
+
+
+CREATE TABLE director_basket_snapshot (
+  basket_uuid bytea CHECK(LENGTH(basket_uuid) = 16) NOT NULL,
+  ts_create bigint NOT NULL,
+  content_checksum bytea CHECK(LENGTH(content_checksum) = 20) NOT NULL,
+  PRIMARY KEY (basket_uuid, ts_create),
+  CONSTRAINT basked_snapshot_basket
+  FOREIGN KEY (basket_uuid)
+    REFERENCES director_basket (uuid)
+    ON DELETE CASCADE
+    ON UPDATE RESTRICT,
+  CONSTRAINT basked_snapshot_content
+  FOREIGN KEY (content_checksum)
+    REFERENCES director_basket_content (checksum)
+    ON DELETE RESTRICT
+    ON UPDATE RESTRICT
+);
+
+CREATE INDEX basket_snapshot_sort_idx ON director_basket_snapshot (ts_create);
+
+
 CREATE TABLE director_generated_config (
   checksum bytea CHECK(LENGTH(checksum) = 20),
   director_version character varying(64) DEFAULT NULL,
@@ -273,6 +314,7 @@ CREATE TABLE icinga_timeperiod (
   zone_id integer DEFAULT NULL,
   object_type enum_object_type_all NOT NULL,
   disabled enum_boolean NOT NULL DEFAULT 'n',
+  prefer_includes enum_boolean DEFAULT NULL,
   PRIMARY KEY (id),
   CONSTRAINT icinga_timeperiod_zone
   FOREIGN KEY (zone_id)
@@ -384,7 +426,7 @@ CREATE TABLE icinga_command (
     ON UPDATE CASCADE
 );
 
-CREATE UNIQUE INDEX command_object_name ON icinga_command (object_name, zone_id);
+CREATE UNIQUE INDEX command_object_name ON icinga_command (object_name);
 CREATE INDEX command_zone ON icinga_command (zone_id);
 COMMENT ON COLUMN icinga_command.object_type IS 'external_object is an attempt to work with existing commands';
 
@@ -1038,6 +1080,25 @@ CREATE TABLE icinga_servicegroup (
 CREATE UNIQUE INDEX servicegroup_object_name ON icinga_servicegroup (object_name);
 CREATE INDEX servicegroup_search_idx ON icinga_servicegroup (display_name);
 
+CREATE TABLE icinga_servicegroup_service_resolved (
+  servicegroup_id integer NOT NULL,
+  service_id integer NOT NULL,
+  PRIMARY KEY (servicegroup_id, service_id),
+  CONSTRAINT icinga_servicegroup_service_resolved_service
+  FOREIGN KEY (service_id)
+    REFERENCES icinga_service (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_servicegroup_service_resolved_servicegroup
+  FOREIGN KEY (servicegroup_id)
+    REFERENCES icinga_servicegroup (id)
+    ON DELETE CASCADE
+    ON UPDATE CASCADE
+);
+
+CREATE INDEX servicegroup_service_resolved_service ON icinga_servicegroup_service_resolved (service_id);
+CREATE INDEX servicegroup_service_resolved_servicegroup ON icinga_servicegroup_service_resolved (servicegroup_id);
+
 
 CREATE TABLE icinga_servicegroup_inheritance (
   servicegroup_id integer NOT NULL,
@@ -1157,6 +1218,11 @@ CREATE TABLE icinga_user (
   FOREIGN KEY (zone_id)
     REFERENCES icinga_zone (id)
     ON DELETE RESTRICT
+    ON UPDATE CASCADE,
+  CONSTRAINT icinga_user_period
+  FOREIGN KEY (period_id)
+    REFERENCES icinga_timeperiod (id)
+    ON DELETE RESTRICT
     ON UPDATE CASCADE
 );
 
@@ -1433,6 +1499,7 @@ CREATE TABLE import_source (
 );
 
 CREATE INDEX import_source_search_idx ON import_source (key_column);
+CREATE UNIQUE INDEX import_source_name ON import_source (source_name);
 
 
 CREATE TABLE import_source_setting (
@@ -1588,6 +1655,7 @@ CREATE TABLE sync_rule (
   PRIMARY KEY (id)
 );
 
+CREATE UNIQUE INDEX sync_rule_name ON sync_rule (rule_name);
 
 CREATE TABLE sync_property (
   id serial,
@@ -1680,10 +1748,9 @@ CREATE TABLE icinga_notification_var (
 );
 
 CREATE INDEX notification_var_command ON icinga_notification_var (notification_id);
-CREATE UNIQUE INDEX notification_var_search_idx ON icinga_notification_var (varname);
+CREATE INDEX notification_var_search_idx ON icinga_notification_var (varname);
 CREATE INDEX notification_var_checksum ON icinga_notification_var (checksum);
 
-
 CREATE TABLE icinga_notification_field (
   notification_id integer NOT NULL,
   datafield_id integer NOT NULL,
@@ -1985,7 +2052,38 @@ CREATE TABLE icinga_dependency_states_set (
 CREATE INDEX dependency_states_set_dependency ON icinga_dependency_states_set (dependency_id);
 COMMENT ON COLUMN icinga_dependency_states_set.merge_behaviour IS 'override: = [], extend: += [], blacklist: -= []';
 
+CREATE TABLE icinga_timeperiod_include (
+  timeperiod_id integer NOT NULL,
+  include_id integer NOT NULL,
+  PRIMARY KEY (timeperiod_id, include_id),
+  CONSTRAINT icinga_timeperiod_timeperiod_include
+  FOREIGN KEY (include_id)
+  REFERENCES icinga_timeperiod (id)
+  ON DELETE RESTRICT
+  ON UPDATE CASCADE,
+  CONSTRAINT icinga_timeperiod_include
+  FOREIGN KEY (timeperiod_id)
+  REFERENCES icinga_timeperiod (id)
+  ON DELETE CASCADE
+  ON UPDATE CASCADE
+);
+
+CREATE TABLE icinga_timeperiod_exclude (
+  timeperiod_id integer NOT NULL,
+  exclude_id integer NOT NULL,
+  PRIMARY KEY (timeperiod_id, exclude_id),
+  CONSTRAINT icinga_timeperiod_timeperiod_exclude
+  FOREIGN KEY (exclude_id)
+  REFERENCES icinga_timeperiod (id)
+  ON DELETE RESTRICT
+  ON UPDATE CASCADE,
+  CONSTRAINT icinga_timeperiod_exclude
+  FOREIGN KEY (timeperiod_id)
+  REFERENCES icinga_timeperiod (id)
+  ON DELETE CASCADE
+  ON UPDATE CASCADE
+);
 
 INSERT INTO director_schema_migration
   (schema_version, migration_time)
-  VALUES (149, NOW());
+  VALUES (158, NOW());
